@@ -2,22 +2,22 @@
 
 $TESTING = true
 
-$: << 'lib'
+$: << "lib"
 
-require 'minitest/autorun'
-require 'ruby2ruby'
-require 'pt_testcase'
-require 'fileutils'
-require 'tmpdir'
-require 'ruby_parser' if ENV["CHECK_SEXPS"]
+require "minitest/autorun"
+require "ruby2ruby"
+require "pt_testcase"
+require "fileutils"
+require "tmpdir"
+require "ruby_parser" if ENV["CHECK_SEXPS"]
 
 class R2RTestCase < ParseTreeTestCase
   def self.previous key
     "ParseTree"
   end
 
-  def self.generate_test klass, node, data, input_name, output_name
-    output_name = data.has_key?('Ruby2Ruby') ? 'Ruby2Ruby' : 'Ruby'
+  def self.generate_test klass, node, data, input_name, _output_name
+    output_name = data.key?("Ruby2Ruby") ? "Ruby2Ruby" : "Ruby"
 
     klass.class_eval <<-EOM
       def test_#{node}
@@ -170,7 +170,7 @@ class TestRuby2Ruby < R2RTestCase
     assert_equal exp, Ruby2Ruby.new.process(sexp)
   end
 
-  def assert_rt src, exp=src.dup
+  def assert_rt src, exp = src.dup
     assert_equal exp, Ruby2Ruby.new.process(RubyParser.new.parse(src))
   end
 
@@ -258,6 +258,29 @@ class TestRuby2Ruby < R2RTestCase
     assert_parse inn, out
   end
 
+  def assert_masgn exp, *args
+    inn = s(:iter, s(:call, nil, :a), s(:args, *args))
+    out = "a { |#{exp}| }"
+    assert_parse inn, out
+  end
+
+  def test_iter_masgn_double_bug
+    assert_masgn("b",
+                 :b)
+    assert_masgn("b, c",
+                 :b, :c)
+    assert_masgn("(b, c)",
+                 s(:masgn, :b, :c))
+    assert_masgn("(b, c), d",
+                 s(:masgn, :b, :c), :d)
+    assert_masgn("b, (c, d), e",
+                 :b, s(:masgn, :c, :d), :e)
+    assert_masgn("(b, (c, d), e), f",
+                 s(:masgn, :b, s(:masgn, :c, :d), :e), :f)
+    assert_masgn("(((b, c), d, e), f), g",
+                 s(:masgn, s(:masgn, s(:masgn, :b, :c), :d, :e), :f), :g)
+  end
+
   def test_attr_writer_double
     inn = s(:defn, :same=, s(:args, :o),
             s(:iasgn, :@same, s(:lvar, :o)), s(:iasgn, :@diff, s(:lvar, :o)))
@@ -274,7 +297,7 @@ class TestRuby2Ruby < R2RTestCase
   def test_attr_writer_same
     do_not_check_sexp!
 
-    inn = s(:defn, :same=, s(:args, :o), s(:iasgn, :@same , s(:lvar, :o)))
+    inn = s(:defn, :same=, s(:args, :o), s(:iasgn, :@same, s(:lvar, :o)))
     out = "attr_writer :same"
     assert_parse inn, out
   end
@@ -420,6 +443,17 @@ class TestRuby2Ruby < R2RTestCase
     assert_parse inn, out
   end
 
+  def test_shadow_block_args
+    inn = s(:iter,
+            s(:call, nil, :a),
+            s(:args,
+              s(:shadow, :b),
+              s(:shadow, :c)))
+    out = 'a { |; b, c| }'
+
+    assert_parse inn, out
+  end
+
   def test_masgn_block_arg
     inn = s(:iter,
             s(:call,
@@ -435,6 +469,42 @@ class TestRuby2Ruby < R2RTestCase
     assert_parse inn, out
   end
 
+  def test_single_nested_masgn_block_arg
+    inn = s(:iter,
+            s(:call, nil, :a),
+            s(:args,
+              s(:masgn,
+                s(:masgn,
+                  s(:masgn, :b)))))
+    out = "a { |(((b)))| }"
+
+    assert_parse inn, out
+  end
+
+  def test_multiple_nested_masgn_block_arg
+    inn = s(:iter,
+            s(:call, nil, :a),
+            s(:args, :b,
+              s(:masgn,
+                s(:masgn, :c, :d),
+                :e,
+                s(:masgn, :f, :g))))
+    out = "a { |b, ((c, d), e, (f, g))| }"
+
+    assert_parse inn, out
+  end
+
+  def test_multiple_nested_masgn_array
+    inn = s(:masgn,
+            s(:array,
+              s(:masgn, s(:array, s(:lasgn, :a), s(:lasgn, :b))),
+              s(:lasgn, :c)),
+            s(:to_ary, s(:call, nil, :fn)))
+    out = "(a, b), c = fn"
+
+    assert_parse inn, out
+  end
+
   def test_masgn_wtf
     inn = s(:block,
             s(:masgn,
@@ -493,7 +563,7 @@ class TestRuby2Ruby < R2RTestCase
               :z,
               s(:lit, 1))
 
-    out ="x&.y&.z(1)"
+    out = "x&.y&.z(1)"
     assert_parse inn, out
   end
 
@@ -762,17 +832,11 @@ class TestRuby2Ruby < R2RTestCase
     rb2 = "a if (not b)"
     rb3 = "a if ! b"
 
-    assert_parse Ruby18Parser.new.parse(rb1), rb1
-    assert_parse Ruby19Parser.new.parse(rb1), rb1
-    assert_parse Ruby25Parser.new.parse(rb1), rb1
+    assert_parse RubyParser.new.parse(rb1), rb1
 
-    assert_parse Ruby18Parser.new.parse(rb2), rb1
-    assert_parse Ruby19Parser.new.parse(rb2), rb1
-    assert_parse Ruby25Parser.new.parse(rb2), rb1
+    assert_parse RubyParser.new.parse(rb2), rb1
 
-    assert_parse Ruby18Parser.new.parse(rb3), rb1
-    assert_parse Ruby19Parser.new.parse(rb3), rb1
-    assert_parse Ruby25Parser.new.parse(rb3), rb1
+    assert_parse RubyParser.new.parse(rb3), rb1
   end
 
   def assert_parse sexp, expected_ruby, expected_eval = nil
@@ -800,10 +864,10 @@ end
 # s
 # t  new    2    3
 
-tr2r = File.read(__FILE__).split(/\n/)[start+1..__LINE__-2].join("\n")
+tr2r = File.read(__FILE__).split(/\n/)[start + 1..__LINE__ - 2].join("\n")
 ir2r = File.read("lib/ruby2ruby.rb")
 
-require 'ruby_parser'
+require "ruby_parser"
 
 def silent_eval ruby
   old, $-w = $-w, nil
@@ -820,9 +884,9 @@ def morph_and_eval src, from, to, processor
 end
 
 unless ENV["SIMPLE"] then
-  ____ = morph_and_eval tr2r, /TestRuby2Ruby/, 'TestRuby2Ruby2', Ruby2Ruby
-  ruby = morph_and_eval ir2r, /Ruby2Ruby/,     'Ruby2Ruby2',     Ruby2Ruby
-  ____ = morph_and_eval ruby, /Ruby2Ruby2/,    'Ruby2Ruby3',     Ruby2Ruby2
+  ____ = morph_and_eval tr2r, /TestRuby2Ruby/, "TestRuby2Ruby2", Ruby2Ruby
+  ruby = morph_and_eval ir2r, /Ruby2Ruby/,     "Ruby2Ruby2",     Ruby2Ruby
+  ____ = morph_and_eval ruby, /Ruby2Ruby2/,    "Ruby2Ruby3",     Ruby2Ruby2
 
   class TestRuby2Ruby1 < TestRuby2Ruby
     def setup
